==> 0.4.0.tar.gz: test/model/field_test.dart <==
@TestGroup(description: 'Field')
library fields.test.model.field;

import 'dart:async';
import 'dart:mirrors';
import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart';
import 'package:fields/model.dart';

main() => reflectTests();

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------

HasFields hasFields;
HasMetadata hasMetadata;
InstanceMirror hasFieldsMirror;
InstanceMirror hasMetadataMirror;
@Setup
void setup() {
  hasFields = new HasFields();
  hasMetadata = new HasMetadata();
  hasFieldsMirror = reflect(hasFields);
  hasMetadataMirror = reflect(hasMetadata);
}

class CustomType {
  int i;
  int get ci => i;
      set ci(int v) => i = v;
  String s;  
  String toString() => 'custom($i,$ci,$s)';
}

class HasFields {
  int i = 7;
  int get ri => 42;
  num n = 3.14;
  final num rn = 6.28;
  double d = 1024.4201;
  final double rd = 1.024;
  String s = 'snarf';
  final String rs = 'immutable';
  List li = [ 1, 2, 3, 4];  
  Map m = { 'a' : 1, 'b' : 2 };
  CustomType c = new CustomType();
}

class HasMetadata {
  @IgnoreField
  int i = 14;
  @UseName('PI')
  num n = 3.14;  
  @UseReadOnly
  int ri = 42;
  @UseDescription('A mutable flag.')
  bool b = false;
}

// -----------------------------------------------------------------------------
// Basic
// -----------------------------------------------------------------------------

@Test('Test the construction of an int field.')
void testInt() {
  final f = new Field.forVariable(hasFieldsMirror, #i);    
  expect(f.value, equals(7));
  expect(f.declaredName, equals(#i));
  expect(f.type, equals(int));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only int field.')
void testIntReadOnly() {
  final f = new Field.forGetter(hasFieldsMirror, #ri);
  expect(f.value, equals(42));    
  expect(f.declaredName, equals(#ri));
  expect(f.type, equals(int));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isTrue);
  expect(f.isVariable, isFalse);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a num field.')
void testNum() {
  final f = new Field.forVariable(hasFieldsMirror, #n);    
  expect(f.value, equals(3.14));  
  expect(f.declaredName, equals(#n));
  expect(f.type, equals(num));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only num field.')
void testNumReadOnly() {
  final f = new Field.forVariable(hasFieldsMirror, #rn);    
  expect(f.value, equals(6.28));  
  expect(f.declaredName, equals(#rn));
  expect(f.type, equals(num));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a double field.')
void testDouble() {
  final f = new Field.forVariable(hasFieldsMirror, #d);    
  expect(f.value, equals(1024.4201));  
  expect(f.declaredName, equals(#d));
  expect(f.type, equals(double));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only double field.')
void testDoubleReadOnly() {
  final f = new Field.forVariable(hasFieldsMirror, #rd);    
  expect(f.value, equals(1.024));  
  expect(f.declaredName, equals(#rd));
  expect(f.type, equals(double));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a string field.')
void testString() {
  final f = new Field.forVariable(hasFieldsMirror, #s);    
  expect(f.value, equals('snarf'));  
  expect(f.declaredName, equals(#s));
  expect(f.type, equals(String));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only string field.')
void testStringReadOnly() {
  final f = new Field.forVariable(hasFieldsMirror, #rs);
  expect(f.value, equals('immutable'));  
  expect(f.declaredName, equals(#rs));
  expect(f.type, equals(String));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a list field.')
void testList() {
  final f = new Field.forVariable(hasFieldsMirror, #li);    
  expect(f.value, equals([1, 2, 3, 4]));  
  expect(f.declaredName, equals(#li));
  expect(f.type, equals(List));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a map field.')
void testMap() {
  final f = new Field.forVariable(hasFieldsMirror, #m);    
  expect(f.value, equals({ 'a' : 1, 'b' : 2 }));  
  expect(f.declaredName, equals(#m));
  expect(f.type, equals(Map));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test that setting a value on a read-only field throws.')
@ExpectError(isStateError)
void testSetValueReadOnlyThrows() {
  final f = new Field.forGetter(hasFieldsMirror, #ri);
  f.value = 13;
}

@Test('Test that setting a value adds a new data event to the stream.')
void testSetValue() {
  final f = new Field.forVariable(hasFieldsMirror, #i);
  int dataCount = 0;  
  f.values.listen(expectAsync1((v) {
    expect(v, equals(8));
  }));  
  f.value = 8;
  expect(f.value, equals(8));
}

@Test('Test that no events are added to the values stream when setting a new'
      ' value if there are no listeners.')
void testSetValueNoValuesListener() {
  final f = new Field.forVariable(hasFieldsMirror, #i);  
  f.value = 8;  
  // Delay to allow the async refresh of children to complete
  new Future.value(f.value).then(expectAsync1((v) {
    expect(v, equals(8));
    final noop = expectAsync0(() {});
    
    // Now attach a listener.
    f.values.listen((_) => fail('No values should be buffered.'));
    
    // Delay test completion to make sure no events are flushed to listener.
    scheduleMicrotask(noop);
  }));
}

// -----------------------------------------------------------------------------
// Metadata
// -----------------------------------------------------------------------------

@Test('Test that factory constructors return null if @IgnoreField is present.')
void testIgnoreField() {
  final f = new Field.forVariable(hasMetadataMirror, #i);
  expect(f, isNull);  
}

@Test('Test that the @UseName annotation is respected.')
void testUseName() {
  final f = new Field.forVariable(hasMetadataMirror, #n);
  expect(f.declaredName, equals(#n));
  expect(f.name, equals('PI'));
  expect(f.annotations, isEmpty);
}

@Test('Test that the @UseDescription annotation is respected.')
void testUseDescription() {
  final f = new Field.forVariable(hasMetadataMirror, #b);
  expect(f.description, equals('A mutable flag.'));
  expect(f.annotations, isEmpty);
}

@Test('Test that the @UseReadOnly annotation is respected.')
void testUseReadOnly() {
  final f = new Field.forVariable(hasMetadataMirror, #ri);
  expect(f.declaredName, equals(#ri));
  expect(f.isReadOnly, isTrue);
}

// -----------------------------------------------------------------------------
// Custom
// -----------------------------------------------------------------------------

@Test('Test the construction of a custom type field.')
void testCustom() {
  final f = new Field.forVariable(hasFieldsMirror, #c);    
  expect(f.value, const isInstanceOf<CustomType>());  
  expect(f.declaredName, equals(#c));
  expect(f.type, equals(CustomType));
  expect(f.isReadOnly, isFalse);
  expect(f.children.length, equals(3));
}

@Test('Test that a child set value affects parent + sibling values.')
void testSetChildValue() {
  final f = new Field.forVariable(hasFieldsMirror, #c);
  
  // Set value listeners on the children
  final i = f.children.firstWhere((c) => c.declaredName == #i);
  i.values.listen(expectAsync1((v) => expect(v, equals(9))));
  
  final ci = f.children.firstWhere((c) => c.declaredName == #ci);
  ci.values.listen(expectAsync1((v) => expect(v, equals(9))));
  
  final s = f.children.firstWhere((c) => c.declaredName == #s);
  s.values.listen((v) => fail('s did not change, expect no stream data.'));

  // Set value listener on the parent
  f.values.listen(expectAsync1((v) => 
      expect('$v', equals('custom(9,9,null)'))));
  
  // Set the value of 'i'
  i.value = 9;
}

==> 0.6.1.tar.gz: test/model/field_test.dart <==
@TestGroup(description: 'Field')
library fields.test.model.field;

import 'dart:async';
import 'dart:mirrors';
import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart' hide Description;
import 'package:fields/model.dart';

main() => reflectTests();

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------

HasFields hasFields;
HasMetadata hasMetadata;
InstanceMirror hasFieldsMirror;
InstanceMirror hasMetadataMirror;
@Setup
void setup() {
  hasFields = new HasFields();
  hasMetadata = new HasMetadata();
  hasFieldsMirror = reflect(hasFields);
  hasMetadataMirror = reflect(hasMetadata);
}

class CustomType {
  int i;
  int get ci => i;
      set ci(int v) => i = v;
  String s;  
  String toString() => 'custom($i,$ci,$s)';
}

class HasFields {
  int i = 7;
  int get ri => 42;
  num n = 3.14;
  final num rn = 6.28;
  double d = 1024.4201;
  final double rd = 1.024;
  String s = 'snarf';
  final String rs = 'immutable';
  List li = [ 1, 2, 3, 4];  
  Map m = { 'a' : 1, 'b' : 2 };
  CustomType c = new CustomType();
}

class HasMetadata {
  @Ignore
  int i = 14;
  @Name('PI')
  num n = 3.14;  
  @ReadOnly
  int ri = 42;
  @Description('A mutable flag.')
  bool b = false;
}

// -----------------------------------------------------------------------------
// Basic
// -----------------------------------------------------------------------------

@Test('Test the construction of an int field.')
void testInt() {
  final f = new Field.forVariable(hasFieldsMirror, #i);    
  expect(f.value, equals(7));
  expect(f.declaredName, equals(#i));
  expect(f.type, equals(int));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only int field.')
void testIntReadOnly() {
  final f = new Field.forGetter(hasFieldsMirror, #ri);
  expect(f.value, equals(42));    
  expect(f.declaredName, equals(#ri));
  expect(f.type, equals(int));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isTrue);
  expect(f.isVariable, isFalse);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a num field.')
void testNum() {
  final f = new Field.forVariable(hasFieldsMirror, #n);    
  expect(f.value, equals(3.14));  
  expect(f.declaredName, equals(#n));
  expect(f.type, equals(num));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only num field.')
void testNumReadOnly() {
  final f = new Field.forVariable(hasFieldsMirror, #rn);    
  expect(f.value, equals(6.28));  
  expect(f.declaredName, equals(#rn));
  expect(f.type, equals(num));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a double field.')
void testDouble() {
  final f = new Field.forVariable(hasFieldsMirror, #d);    
  expect(f.value, equals(1024.4201));  
  expect(f.declaredName, equals(#d));
  expect(f.type, equals(double));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only double field.')
void testDoubleReadOnly() {
  final f = new Field.forVariable(hasFieldsMirror, #rd);    
  expect(f.value, equals(1.024));  
  expect(f.declaredName, equals(#rd));
  expect(f.type, equals(double));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a string field.')
void testString() {
  final f = new Field.forVariable(hasFieldsMirror, #s);    
  expect(f.value, equals('snarf'));  
  expect(f.declaredName, equals(#s));
  expect(f.type, equals(String));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a read-only string field.')
void testStringReadOnly() {
  final f = new Field.forVariable(hasFieldsMirror, #rs);
  expect(f.value, equals('immutable'));  
  expect(f.declaredName, equals(#rs));
  expect(f.type, equals(String));
  expect(f.isReadOnly, isTrue);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a list field.')
void testList() {
  final f = new Field.forVariable(hasFieldsMirror, #li);    
  expect(f.value, equals([1, 2, 3, 4]));  
  expect(f.declaredName, equals(#li));
  expect(f.type, equals(List));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test the construction of a map field.')
void testMap() {
  final f = new Field.forVariable(hasFieldsMirror, #m);    
  expect(f.value, equals({ 'a' : 1, 'b' : 2 }));  
  expect(f.declaredName, equals(#m));
  expect(f.type, equals(Map));
  expect(f.isReadOnly, isFalse);
  expect(f.isGetter, isFalse);
  expect(f.isVariable, isTrue);
  expect(f.children, isEmpty);
}

@Test('Test that setting a value on a read-only field throws.')
@ExpectError(isStateError)
void testSetValueReadOnlyThrows() {
  final f = new Field.forGetter(hasFieldsMirror, #ri);
  f.value = 13;
}

@Test('Test that setting a value adds a new data event to the stream.')
void testSetValue() {
  final f = new Field.forVariable(hasFieldsMirror, #i);
  int dataCount = 0;  
  f.values.listen(expectAsync1((v) {
    expect(v, equals(8));
  }));  
  f.value = 8;
  expect(f.value, equals(8));
}

@Test('Test that no events are added to the values stream when setting a new'
      ' value if there are no listeners.')
void testSetValueNoValuesListener() {
  final f = new Field.forVariable(hasFieldsMirror, #i);  
  f.value = 8;  
  // Delay to allow the async refresh of children to complete
  new Future.value(f.value).then(expectAsync1((v) {
    expect(v, equals(8));
    final noop = expectAsync0(() {});
    
    // Now attach a listener.
    f.values.listen((_) => fail('No values should be buffered.'));
    
    // Delay test completion to make sure no events are flushed to listener.
    scheduleMicrotask(noop);
  }));
}

// -----------------------------------------------------------------------------
// Metadata
// -----------------------------------------------------------------------------

@Test('Test that factory constructors return null if @IgnoreField is present.')
void testIgnoreField() {
  final f = new Field.forVariable(hasMetadataMirror, #i);
  expect(f, isNull);  
}

@Test('Test that the @UseName annotation is respected.')
void testUseName() {
  final f = new Field.forVariable(hasMetadataMirror, #n);
  expect(f.declaredName, equals(#n));
  expect(f.name, equals('PI'));
  expect(f.annotations, isEmpty);
}

@Test('Test that the @UseDescription annotation is respected.')
void testUseDescription() {
  final f = new Field.forVariable(hasMetadataMirror, #b);
  expect(f.description, equals('A mutable flag.'));
  expect(f.annotations, isEmpty);
}

@Test('Test that the @UseReadOnly annotation is respected.')
void testUseReadOnly() {
  final f = new Field.forVariable(hasMetadataMirror, #ri);
  expect(f.declaredName, equals(#ri));
  expect(f.isReadOnly, isTrue);
}

// -----------------------------------------------------------------------------
// Custom
// -----------------------------------------------------------------------------

@Test('Test the construction of a custom type field.')
void testCustom() {
  final f = new Field.forVariable(hasFieldsMirror, #c);    
  expect(f.value, const isInstanceOf<CustomType>());  
  expect(f.declaredName, equals(#c));
  expect(f.type, equals(CustomType));
  expect(f.isReadOnly, isFalse);
  expect(f.children.length, equals(3));
}

@Test('Test that a child set value affects parent + sibling values.')
void testSetChildValue() {
  final f = new Field.forVariable(hasFieldsMirror, #c);
  
  // Set value listeners on the children
  final i = f.children.firstWhere((c) => c.declaredName == #i);
  i.values.listen(expectAsync1((v) => expect(v, equals(9))));
  
  final ci = f.children.firstWhere((c) => c.declaredName == #ci);
  ci.values.listen(expectAsync1((v) => expect(v, equals(9))));
  
  final s = f.children.firstWhere((c) => c.declaredName == #s);
  s.values.listen((v) => fail('s did not change, expect no stream data.'));

  // Set value listener on the parent
  f.values.listen(expectAsync1((v) => 
      expect('$v', equals('custom(9,9,null)'))));
  
  // Set the value of 'i'
  i.value = 9;
}
